#include <stdio.h>
#include <stdlib.h>
#include "libusb-1.0/libusb.h"

int main(int argc, char *argv[])
{
	int status;

	if (0 != libusb_init(NULL))
	{
		printf("libusb_init() failed\n");
		return -1;
	}

	libusb_device_handle *devH = libusb_open_device_with_vid_pid(NULL, 0x3231, 0x0100);
	if (!devH)
	{
		printf("libusb open failed\n");
		return -1;
	}

	status = libusb_set_configuration(devH, 1);
	if (status)
	{
		printf("libusb_set_configuration() failed: %s\n", libusb_strerror((enum libusb_error)status));
		return -1;
	}

	status = libusb_claim_interface(devH, 0);
	if (status)
	{
		printf("libusb_claim_interface() failed: %s\n", libusb_strerror((enum libusb_error)status));
		return -1;
	}

	printf("Kneron KL520 usb device open OK\n");

	int buf_len = 4 * 1024;
	unsigned char *buf = (unsigned char *)malloc(buf_len);
    unsigned int *data_ptr = (unsigned int*)buf;
    unsigned int count = 0;
    int i;

	while(1)
	{
		int transferred;

        count = 0;
		status = libusb_bulk_transfer(devH, 0x81, buf, buf_len, &transferred, 1000000);
		if (status != 0)
		{
			printf("bulk in transfer failed error: %d\n", status);
			break;
		}

        count =  *data_ptr;

        if(count) {
            printf("BBox count = %d\n", count);

            for ( i = 0;i < count;i++) {
                printf("BBox[%d] = (%d,%d),(%d,%d)\n", i,
                    data_ptr[1+i*4],
                    data_ptr[1+i*4+1],
                    data_ptr[1+i*4+2],
                    data_ptr[1+i*4+3]);
            }
            printf("\n");
            fflush(stdout);
        }
	}

	free(buf);

	libusb_exit(NULL);

	return 0;
}
