/*
 * Kneron memory APIs
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */

#ifndef __KDP_MEMORY_H_
#define __KDP_MEMORY_H_

#include <stdint.h>

/**
 * @brief To initialize available DDR block 
 * @param start_addr the start address of DDR block(bigger address)
 * @param end_addr the end address of DDR block(smaller address)
 */
void        kmdw_ddr_init(uint32_t start_addr, uint32_t end_addr);


/**
 * @brief to set ddr boundary address(smaller address)
 * @param [in] boundary the address of boundary]
 * @return 0: ok,  -1: failed(cross available boundary)
 * @notes
 *   once boundary address is updated, all reserved blocks 
 *   might become invalid with the new boundary
 *   !!warning!! be sure you well know the boundary is safe to update
 */
int kmdw_ddr_set_ddr_boundary(uint32_t boundary);


/**
 * @brief to allocate DDR memory
 * @param numbtye size in byte
 * @return the address of allocated block
 */
uint32_t    kmdw_ddr_reserve(uint32_t numbyte);


/**
 * @brief to allocate DDR memory in reverse direction 
 * @param[in] numbyte size in byte
 * @return the address of allocated block
 */
uint32_t kmdw_ddr_reserve_in_reverse(uint32_t numbyte);

#endif

