#include <string.h>
#include "kmdw_memory.h"
#include "kmdw_console.h"

/* ddr malloc direction : from tail(bigger address) to head (smaller) */

static uint32_t s_ddr_addr_tail = 0;
static uint32_t s_ddr_addr_boundary = 0;

#define ALIGN_BYTE 16
static uint32_t _align(uint32_t n);
    
uint32_t _align(uint32_t n) {
    return (n + ALIGN_BYTE - 1) & ~( ALIGN_BYTE - 1);
}

void kmdw_ddr_init(uint32_t start_addr, uint32_t end_addr)
{
    s_ddr_addr_boundary = end_addr; //(lower addr) KDP_DDR_MODEL_RESERVED_END
    s_ddr_addr_tail = start_addr;   //(higher addr)KDP_DDR_HEAP_HEAD_FOR_MALLOC+1
}

int kmdw_ddr_set_ddr_boundary(uint32_t boundary)
{
    if (boundary >= s_ddr_addr_tail)
        return -1;
    else {
        s_ddr_addr_boundary = boundary;
        return 0;
    }
}

uint32_t kmdw_ddr_reserve_in_reverse(uint32_t numbyte)
{
    uint32_t aligned_numbyte;
    if(numbyte == 0)
        return 0;
    if(s_ddr_addr_boundary == 0)
        return 0; //not initialized yet

    aligned_numbyte = _align(numbyte);

    uint32_t old_addr = s_ddr_addr_boundary;
    uint32_t new_addr = s_ddr_addr_boundary;
    new_addr += aligned_numbyte;

    if(new_addr >= (s_ddr_addr_tail-1))
    {
        err_msg("[ERR] [%s] fail allocation: %8d byte [ 0x%x -> 0x%x >= 0x%x]\n",
            __FUNCTION__, numbyte, old_addr, new_addr, (s_ddr_addr_tail - 1));
        return 0;
    }
    else
    {
        dbg_msg("[DBG] [%s] allocated %8d [ *0x%x -> 0x%x]\n",
            __FUNCTION__, numbyte, old_addr, new_addr);

        s_ddr_addr_boundary = new_addr;
        return old_addr+1;
    }
}

uint32_t kmdw_ddr_reserve(uint32_t numbyte)
{
    uint32_t aligned_numbyte;
    if(numbyte == 0)
        return 0;

    if(s_ddr_addr_boundary == 0)
        return 0; //not initialized yet

    aligned_numbyte = _align(numbyte);

    uint32_t new_tail = s_ddr_addr_tail;
    new_tail -= aligned_numbyte;

    if(new_tail <= (s_ddr_addr_boundary+1))
    {
        err_msg("[ERR] [%s] fail allocation: %8d byte [ 0x%x(<=0x%x) <- 0x%x]\n",
            __FUNCTION__, numbyte, new_tail, (s_ddr_addr_boundary+1), s_ddr_addr_tail);

        return 0; 
    }
    else
    {
        dbg_msg("[DBG] [%s] allocated %8d [ *0x%x <- 0x%x]\n",
            __FUNCTION__, numbyte, new_tail, s_ddr_addr_tail);

        s_ddr_addr_tail = new_tail;
        return s_ddr_addr_tail;
    }
}
