/* -------------------------------------------------------------------------- 
 * Copyright (c) 2013-2016 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *      Name:    main.c
 *      Purpose: RTX for Kneron
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "kdrv_uart.h"
#include "kdrv_gpio.h"

#include "cmsis_os2.h"                    // ARM::CMSIS:RTOS2:Keil RTX5

#include "kdrv_system.h"
#include "rtc.h"
#include "kmdw_console.h"


#include "kdrv_pwm.h"
#include "kdrv_ddr.h"
#include "kdp_ddr_table.h"  //for default ddr table
#include "pinmux.h"
#include "kdrv_ssp.h"
#include "kdrv_pinmux.h"
#include "kdrv_clock.h"

#define DDR1_DST    0x60000000
#define FLASH_SRC   0x00
#define LENGTH      (1024*1024*5)
#define DDR2_DST    (DDR1_DST+LENGTH)

#undef err_msg
#define err_msg kmdw_printf
#define _get_min(x,y) ( x < y ? x: y )

extern struct st_ssp_spi	driver_ssp_ctx;
#if(SSP_SPI_MASTER_DEV==COM_BUS_TYPE_SSP1 || SSP_SPI_MASTER_DEV==COM_BUS_TYPE_SSP0)
extern struct st_ssp_spi driver_ssp_master_ctx;
#endif
static void kmdw_ssp_spi1_callback(uint32_t event);
//---
uint8_t kmdw_ssp_api_spi_init(kdrv_ssp_spi_dev_id_t handle, enum e_spi edata)
{
    if( kdrv_ssp_statemachine( handle, &driver_ssp_ctx, edata, kmdw_ssp_spi1_callback ) == e_spi_ret_init_done ){
        return 1;
    }
    return 0;
}

uint8_t kmdw_ssp_api_spi_enable(kdrv_ssp_spi_dev_id_t handle, struct st_ssp_spi *stspi)
{
    if( kdrv_ssp_statemachine( handle, stspi, e_spi_enable, NULL ) == e_spi_ret_enable_done ){
        return 1;
    }
    return 0;
}

uint8_t kmdw_ssp_api_spi_disable(kdrv_ssp_spi_dev_id_t handle, struct st_ssp_spi *stspi)
{
    if( kdrv_ssp_statemachine( handle, stspi, e_spi_disable, NULL ) == e_spi_ret_disableDone ){
        return 1;
    }
    return 0;
}

uint8_t kmdw_ssp_api_spi_receive(kdrv_ssp_spi_dev_id_t handle, struct st_ssp_spi *stspi)
{
    if( kdrv_ssp_statemachine( handle, stspi, e_spi_rx, NULL ) == e_spi_ret_rxbusy ){
        return 0;		//rx on-going
    }
    else{
        return 1;		//rx done
    }
}

uint8_t kmdw_ssp_api_spi_receive_xor(kdrv_ssp_spi_dev_id_t handle)
{
    if( kdrv_ssp_statemachine( handle, &driver_ssp_ctx, e_spi_rx_check, NULL ) == e_spi_ret_rx_xor_OK ){
            return 1;		//rx data correct
    }
    else{
            return 0;		//rx data fail
    }
}

uint8_t kmdw_ssp_api_spi_transfer(kdrv_ssp_spi_dev_id_t handle)
{
    if( kdrv_ssp_statemachine( handle, &driver_ssp_ctx, e_spi_tx, NULL ) == e_spi_ret_txbusy ){
            return 1;		//rx data correct
    }
    else{
            return 0;		//rx data fail
    }
}

uint8_t kmdw_ssp_api_spi_transfer_checks(kdrv_ssp_spi_dev_id_t handle)
{
    if( kdrv_ssp_statemachine( handle, &driver_ssp_ctx, e_spi_tx_status_check, NULL ) == e_spi_ret_txbusy )
    {
            return 0;
    }
    else
    {
            return 1;
    }
}

void kmdw_ssp_api_spi_write_tx_buff( uint8_t *src, uint16_t nlen )
{
    kdrv_ssp_write_buff( &driver_ssp_ctx, src, nlen );
}

/* This is a spi master tx/rx loopback test example 
 * regSSP0_ctrl(driver_ssp_ctx.port_no)->st.bf.kdrv_ssp_sspcr0.LBM to enable loopback mode
*/
uint8_t nsize = 0;
static void ssp_spi_slave_thread(void *argument)
{
    err_msg(" ssp_spi_slave_test : Init\n");
    if( kmdw_ssp_api_spi_init(DEBUG_SPI_PORT, e_spi_init_slave) != 1 )
    {
        err_msg(" ssp_spi_slave_test : error!!\n");
        return;
    }
    regSSP0_ctrl(driver_ssp_ctx.port_no)->st.bf.kdrv_ssp_sspcr0.LBM=0; //please set to 0 if not test in loop back mode
    kmdw_ssp_api_spi_init(DEBUG_SPI_PORT, e_spi_txrx_reinit);
    kmdw_ssp_api_spi_enable(DEBUG_SPI_PORT, &driver_ssp_ctx);

    memset((void *)driver_ssp_ctx.Rx_buffer, 0x55, 300);
    err_msg(" ssp_spi_slave_test : Start\n");
    driver_ssp_ctx.pre_size = 40;
    while(1)
    {
        if(driver_ssp_ctx.interrupt_en == 0x00)
        {
            kdrv_ssp_rx_polling_receive_all(&driver_ssp_ctx);
            /*nsize = driver_ssp_ctx.Rx_buffer_index;
            if(nsize)
            {
                //ssp_write_word(SSP_REG_BASE_S, &driver_ssp_ctx.Rx_buffer ,k);
                err_msg("\nrx data ");
                for(int i=0; i<nsize; i++)
                {
                    err_msg(" 0x%X",*( driver_ssp_ctx.Rx_buffer+ i ));
                }
                err_msg("\n");
            }*/
        }
        if(driver_ssp_ctx.interrupt_en == 0x04)
        {
            int32_t flags;
            flags = osThreadFlagsWait(FLAG_SPI1_RX_DONE, osFlagsWaitAny, osWaitForever);
            osThreadFlagsClear(flags);
            err_msg("\nrx data ");
            for(int i=0; i<driver_ssp_ctx.pre_size; i++)
            {
                err_msg(" 0x%X",*( driver_ssp_ctx.Rx_buffer+ i ));
            }
            err_msg("\n");
        }
    }
}

static osThreadId_t tid_spi_com;
static void kmdw_ssp_spi1_callback(uint32_t event)
{
    if (event & ARM_SPI_EVENT_RECEIVE_COMPLETE)
    {
        osThreadFlagsSet(tid_spi_com, FLAG_SPI1_RX_DONE);
    }

    if (event & ARM_SPI_EVENT_SEND_COMPLETE)
    {
        osThreadFlagsSet(tid_spi_com, FLAG_SPI1_TX_DONE);
    }

    if (event & ARM_SPI_EVENT_RX_TIMEOUT)
    {
        osThreadFlagsSet(tid_spi_com, FLAG_SPI1_RX_TIMEOUT);
    }

}

/**
 * @brief main, main dispatch function
 */
int main(void) 
{
    SystemCoreClockUpdate();            // System Initialization
    osKernelInitialize();               // Initialize CMSIS-RTOS

    /* below is some primiary system init settings */
    kdrv_system_init(); // primary system init
    /* below is to enable SSP0/1 clock settings */
    kdrv_clock_enable(CLK_SSP1_1_SSPCLK);
    kdrv_clock_enable(CLK_SSP1_0_SSPCLK);
    kdrv_clock_enable(CLK_SSP0_1_SSPCLK);
    kdrv_clock_enable(CLK_SSP0_0_SSPCLK);

    kdrv_system_init_ncpu(); // this is a must for ddr
    kdrv_ddr_system_init(DDR_INIT_ALL); // enable ddr
    kmdw_ddr_init(DDR2_DST,DDR1_DST);

    /* below is some middleware init settings */
    kdrv_uart_initialize(); // for log printing
    kmdw_console_init(UART0_DEV);   // enable console mdw for printing
    kdrv_pinmux_init();
#if ( SPI_SLAVE_PORT_DEV == COM_BUS_TYPE_SSP0 )
    uint32_t data;
    data = ( inw(SCU_EXTREG_PA_BASE + 0x14C) &0xFFFFFFF8) | (1<< 6) | 1<<3;
    outw(SCU_EXTREG_PA_BASE + 0x14C, data | 0x4) ;
    data = ( inw(SCU_EXTREG_PA_BASE + 0x150) &0xFFFFFFF8) | (1<< 6) | 1<<3;
    outw(SCU_EXTREG_PA_BASE + 0x150, data | 0x4);
    data = ( inw(SCU_EXTREG_PA_BASE + 0x154) &0xFFFFFFF8) | (1<< 6) | 1<<3;
    outw(SCU_EXTREG_PA_BASE + 0x154, data | 0x4);
    data = ( inw(SCU_EXTREG_PA_BASE + 0x158) &0xFFFFFFF8) | (1<< 6) | 1<<3;
    outw(SCU_EXTREG_PA_BASE + 0x158, data | 0x4);

#elif ( SPI_SLAVE_PORT_DEV == COM_BUS_TYPE_SSP1 )
    uint32_t data;
    data = (inw(SCU_EXTREG_PA_BASE + 0x15c)&0xFFFFFFF8) | (1<< 6) | 1<<3;
    outw(SCU_EXTREG_PA_BASE + 0x15c, data | 0x2) ;
    //cs pin as GPIO output
    data = ( inw(SCU_EXTREG_PA_BASE + 0x160)&0xFFFFFFF8) | (1<< 6) | 1<<3;
    outw(SCU_EXTREG_PA_BASE + 0x160, data | 0x3); //as gpio
    //kdrv_gpio_set_attribute( GPIO_PIN_20, GPIO_DIR_OUTPUT );
    //kdrv_ssp_spi_CS_set(1);
    data = ( inw(SCU_EXTREG_PA_BASE + 0x164)&0xFFFFFFF8) | (1<< 6) | 1<<3;
    outw(SCU_EXTREG_PA_BASE + 0x164, data | 0x2);
    data = (inw(SCU_EXTREG_PA_BASE + 0x168) &0xFFFFFFF8) | (1<< 6) | 1<<3;
    outw(SCU_EXTREG_PA_BASE + 0x168, data | 0x2);
#endif
    
    tid_spi_com = osThreadNew(ssp_spi_slave_thread, NULL, NULL);;

    //application is triggered in host_com.c
    if (osKernelGetState() == osKernelReady)
    {
        osKernelStart();
    }

    while (1)
    {
    }
}


